﻿//////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011 Dynastream Innovations Inc.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
//////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text; 
using System.Windows;
using System.Reflection;
using System.Windows.Forms;
using ANT_Managed_Library;
using System.Threading;
using System.IO;


namespace Frequency_Agility_Demo
{
    delegate void dPrint(String msg);
    delegate void dSetTextCallback(string s);
    delegate void dSetButtonText();

    public partial class MainForm : Form
    {
        //Path to store log files in (if turned on). This puts them into the same directory as the executable
        String logDirectory = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

        //Device Connecting Parameters
        private static readonly ushort usBaudRateAP2AP3 = 57600; //AP2 and AP3 Baud Rate
        private static readonly ushort usBaudRateAP1 = 50000; //AP1 Baud Rate
        private byte ucUSBPort1;
        private byte ucUSBPort2;

        //ANT Channel Configuration Parameters
        private ushort usMasterNo = 33; //Master Device Number (ANT Channel ID)
        private ushort usSlaveNo = 33; //Slave Device  Number (ANT Channel ID)
        private byte ucMasterType = 1; //Master Device Type (ANT Channel ID)
        private byte ucSlaveType = 1; //Slave Device Type (ANT Channel ID)
        private byte ucMasterTrans = 1; //Master Transmission Type (ANT Channel ID)
        private byte ucSlaveTrans = 1; //Slave Transmission Type (ANT Channel ID)
        private byte[] aucMasterNetworkKey = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }; //Master Network Key (Default public network)
        private byte[] aucSlaveNetworkKey = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }; //Slave Network Key (Default public network)
        private byte ucNetworkNo = 0; //Network Number 
        private byte ucMasterRF = 3; //Master Radio Frequency 
        private byte ucSlaveRF = 3; //Slave Radio Frequency
        private ushort usMasterChannelPeriod = 8192; //Master Channel Period T/32768 seconds
        private ushort usSlaveChannelPeriod = 8192; //Slave Channel Period T/32768 seconds
        private double dMasterChannelFrequency = 4; //Master Channel Period in Hz

        //Frequency Agility Parameters
        private byte ucMasterFreq1 = 3; //Master Frequency Agility Frequency 1
        private byte ucSlaveFreq1 = 3; //Slave Frequency Agility Frequency 1
        private byte ucMasterFreq2 = 39; //Master Frequency Agility Frequency 2
        private byte ucSlaveFreq2 = 39; //Slave Frequency Agility Frequency 2
        private byte ucMasterFreq3 = 75; //Master Frequency Agility Frequency 3
        private byte ucSlaveFreq3 = 75; //Slave Frequency Agility Frequency 3

        //ANT Devices
        private static ANT_Device theMasterDevice; //Master ANT Device
        private static ANT_Device theSlaveDevice; //Slave ANT Device

        //ANT Channels
        private static ANT_ReferenceLibrary.ChannelType _masterChannelType = ANT_ReferenceLibrary.ChannelType.BASE_Master_Transmit_0x10; //Master Channel Assignment 
        private ANT_Channel _channel0; //Master ANT Channel
        private static ANT_ReferenceLibrary.ChannelType _slaveChannelType = ANT_ReferenceLibrary.ChannelType.BASE_Slave_Receive_0x00; //Slave Channel Assignment
        private ANT_Channel _channel1; //Slave ANT Channel

        //ANT Message
        private byte[] aucTxBuffer = new byte[] { 0, 0, 0, 0, 0, 0, 0, 0 }; //Acknowledge Message 
        
        //Main Form Initialization Parameters
        private string[] astrAssignUSBPort = new string[] { "0", "1", "2", "3" };//USB Port Numbers
        private string strNetworkKey = "00-00-00-00-00-00-00-00"; //Network Key String

        //Channel Response Parameters
        private ushort usMasterSearchCount; //Number of allowable missed messages for the Master based on the Channel Period and Search Timeout (Period (Hz) * (Search timeout / 3))
        private ushort usSlaveSearchCount; //Used to determine Slave Search Timeout: Slave Search Count * 2.5 = Search Timeout (s)
        private double dMasterSearchTimeout = 30; //Master Search Timeout (High Priority Only)
        private double dSlaveSearchTimeout = 30; //Slave Search Timeout (High Priority Only)
        private ushort usMasterCounter = 1; //Counter. Increments after every consecutive missed Acknowledge message
        private byte ucSlaveReceived = 0; //Used to Determine if the Slave has Received Messages on the Current Radio Frequency
        private ushort usNoConnection = 0; //Counter. Increments after every RX_FAIL event. Determines when AP1 goes to search
        private byte ucSlaveSearchTimeoutCount = 0; //Counter. Determine When Slave Channel Should Close. Maximum Times Slave Searches, 6
        private ushort usSlaveMissedRx; //Maximum Allowable RX_FAIL events
        private bool bSlaveClosed = false; //True if Close Channel Button Clicked, or 6 Consecutive Search Timeouts on Slave channel

        //Module Type
        private bool bAP1 = false; //True if an AP1 device is detected (Slave channel response only)
        private bool bAT3 = false; //True if an AT3 defice is detected (Slave channel response only)




        /// <summary>
        /// Create and start Frequency Agility demo application
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            //Display the version number on the upper left hand corner of the application form
            this.Text += " - v." + Assembly.GetExecutingAssembly().GetName().Version.Major + "." + Assembly.GetExecutingAssembly().GetName().Version.Minor; 
            //Locks the minimum form size to the current designer's size
            this.MinimumSize = this.Size; 
            //Enables ANT device log files, to turn off, comment out  
            EnableLogging(); 
        }

        /// <summary>
        /// Initialize and load the Frequency Agility demo application
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="args">Event arguments</param>
        private void Form1_Load(object sender, EventArgs e)
        {
            //Set application to start state
            MasterStartState();
            SlaveStartState();

            //Clear any items that may be in the USB combo boxes
            usbComboBox1.Items.Clear();
            usbComboBox2.Items.Clear();

            //Populate the USB combo boxes
            usbComboBox1.Items.AddRange(astrAssignUSBPort);
            usbComboBox2.Items.AddRange(astrAssignUSBPort);

            //Set Channel Period defaults
            masterSeconds.Text = "8192";
            slaveSeconds.Text = "8192";
            masterHz.Text = "4";
            slaveHz.Text = "4";

            //Set Network Key to default
            networkKeyTextBox1.Text = strNetworkKey; 
            networkKeyTextBox2.Text = strNetworkKey;

            //Set Search Count to default
            masterSearchUpDown.Value = 12;
            slaveSearchUpDown.Value = 12;

            //Set and Display default search timeout in seconds
            dMasterSearchTimeout = (double)masterSearchUpDown.Value * 2.5;
            masterSearchTextBox.Text = dMasterSearchTimeout.ToString() + " s";
            dSlaveSearchTimeout = (double)slaveSearchUpDown.Value * 2.5;
            slaveSearchTextBox.Text = dSlaveSearchTimeout.ToString() + " s";

            //Set the Search Count variables to default 
            usMasterSearchCount = (ushort) ((32768 / usMasterChannelPeriod) * (dMasterSearchTimeout/3));
            usSlaveSearchCount = (ushort)((32768 / usSlaveChannelPeriod) * dSlaveSearchTimeout);
            
            //set the default maximum missed messages for the slave
            usSlaveMissedRx = (ushort)((65535/usSlaveChannelPeriod) +1);
        }

        /// <summary>
        /// Handle USB Port index changes
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="args">Event arguments</param>
        private void usbComboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            ucUSBPort1 = (byte)usbComboBox1.SelectedIndex; 
        }
        private void usbComboBox2_SelectedIndexChanged(object sender, EventArgs e)
        {
            ucUSBPort2 = (byte)usbComboBox2.SelectedIndex; 
        }

        /// <summary>
        /// Handle changes to the Master Channel ID
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void deviceNo1_ValueChanged(object sender, EventArgs e)
        {
            usMasterNo = (ushort)deviceNo1.Value; 
        }
        private void deviceType1_ValueChanged(object sender, EventArgs e)
        {
            ucMasterType = (byte)deviceType1.Value;
        }
        private void transType1_ValueChanged(object sender, EventArgs e)
        {
            ucMasterTrans = (byte)transType1.Value;
        }

        /// <summary>
        /// Handle changes to the Slave Channel ID
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void deviceNo2_ValueChanged(object sender, EventArgs e)
        {
            usSlaveNo = (ushort)deviceNo2.Value;
        }
        private void deviceType2_ValueChanged(object sender, EventArgs e)
        {
            ucSlaveType = (byte)deviceType2.Value;
        }
        private void transType2_ValueChanged(object sender, EventArgs e)
        {
            ucSlaveTrans = (byte)transType2.Value; 
        }

        /// <summary>
        /// Handle the channel period display and conversion between message rate and period for the Master
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void masterSeconds_TextChanged(object sender, EventArgs e)
        {
            if (!masterSeconds.Focused && sender != this)
                return;
            try
            {
                ushort usSeconds = ushort.Parse(masterSeconds.Text);
                double dHz = Math.Round((double)32768 / usSeconds, 3);
                String sRounded = dHz.ToString();
                //handle the rounding, so that it fits in the display
                if (sRounded.Length > 5 && !sRounded.Equals("Infinity"))
                {
                    if (sRounded[4] == '.')
                        sRounded = sRounded.Remove(4);
                    else
                        sRounded = sRounded.Remove(5);
                }

                masterHz.Text = sRounded;
                usMasterChannelPeriod = ushort.Parse(masterSeconds.Text);
                dMasterChannelFrequency = double.Parse(masterHz.Text);
            }

            catch (Exception)
            {
                masterHz.Text = "error!";
            }
        }
        private void masterHz_TextChanged(object sender, EventArgs e)
        {
            if (!masterHz.Focused)
                return;

            try
            {
                double dHz = double.Parse(masterHz.Text);
                if (dHz < 0.5 || dHz > 32768)
                    throw new Exception("Period out of range");
                ushort usSeconds = (ushort)Math.Round(32768 / dHz);
                if (usSeconds == 0)//0 only occurs if it is on a rollover past 65535
                    usSeconds = 65535;
                masterSeconds.Text = usSeconds.ToString();
                dMasterChannelFrequency = double.Parse(masterHz.Text);
                usMasterChannelPeriod = ushort.Parse(masterSeconds.Text);
            }

            catch (Exception)
            {
                masterSeconds.Text = "error";
            }
        }

        /// <summary>
        /// Handle the channel period display and conversion between message rate and period for the Slave
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void slaveSeconds_TextChanged(object sender, EventArgs e)
        {
            if (!slaveSeconds.Focused && sender != this)
                return;
            try
            {
                ushort usSeconds = ushort.Parse(slaveSeconds.Text);
                double dHz = Math.Round((double)32768 / usSeconds, 3);
                String sRounded = dHz.ToString();
                //Handle rounding
                if (sRounded.Length > 5 && !sRounded.Equals("Infinity"))
                {
                    if (sRounded[4] == '.')
                        sRounded = sRounded.Remove(4);
                    else
                        sRounded = sRounded.Remove(5);
                }

                slaveHz.Text = sRounded;
                usSlaveChannelPeriod = ushort.Parse(slaveSeconds.Text);
                if (usSlaveChannelPeriod > 29789)
                    usSlaveMissedRx = 4;
                else
                    usSlaveMissedRx = (ushort)((65535 / usSlaveChannelPeriod) + 1);
            }

            catch (Exception)
            {
                slaveHz.Text = "error!";
            }
        }
        private void slaveHz_TextChanged(object sender, EventArgs e)
        {
            if (!slaveHz.Focused)
                return;

            try
            {
                double dHz = double.Parse(slaveHz.Text);
                if (dHz < 0.5 || dHz > 32768)
                    throw new Exception("Period out of range");
                ushort usSeconds = (ushort)Math.Round(32768 / dHz);
                if (usSeconds == 0)//0 only occurs if it is on a rollover past 65535
                    usSeconds = 65535;
                slaveSeconds.Text = usSeconds.ToString();
                usSlaveChannelPeriod= ushort.Parse(slaveSeconds.Text);
                if (usSlaveChannelPeriod > 29789)
                    usSlaveMissedRx = 4;
                else
                    usSlaveMissedRx = (ushort)((65535 / usSlaveChannelPeriod) + 1);
            }

            catch (Exception)
            {
                slaveSeconds.Text = "error";
            }
        }

        /// <summary>
        /// Handle changes to the frequency agility channel frequencies for the Master
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void freq11_ValueChanged(object sender, EventArgs e)
        {
            ucMasterFreq1 = (byte)freq11.Value;
            //Update initial RF value 
            ucMasterRF = ucMasterFreq1;
        }
        private void freq21_ValueChanged(object sender, EventArgs e)
        {
            ucMasterFreq2 = (byte)freq21.Value;
        }
        private void freq31_ValueChanged(object sender, EventArgs e)
        {
            ucMasterFreq3 = (byte)freq31.Value;
        }

        /// <summary>
        /// Hanndle changes to the frequency agility channel frequencies for the Slave
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void freq12_ValueChanged(object sender, EventArgs e)
        {
            ucSlaveFreq1 = (byte)freq12.Value;
            //Update the initial RF Frequency
            ucSlaveRF = ucSlaveFreq1;
        }
        private void freq22_ValueChanged(object sender, EventArgs e)
        {
            //change the value of the second frequency agility channel frequency 
            ucSlaveFreq2 = (byte)freq22.Value; 
        }
        private void freq32_ValueChanged(object sender, EventArgs e)
        {
            //change the value of the third frequency agility channel 
            ucSlaveFreq3 = (byte)freq32.Value;
        }

        /// <summary>
        /// Handle changes to the Search Timeout Counts and update the Search Timeout in Seconds
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void masterSearchUpDown_ValueChanged(object sender, EventArgs e)
        {
            usMasterSearchCount = (byte)masterSearchUpDown.Value;
            dMasterSearchTimeout = ((double)usMasterSearchCount * 2.5);
            masterSearchTextBox.Text = dMasterSearchTimeout.ToString() + " s";
        }
        private void slaveSearchUpDown_ValueChanged(object sender, EventArgs e)
        {
            usSlaveSearchCount = (byte)slaveSearchUpDown.Value;
            dSlaveSearchTimeout = ((double)usSlaveSearchCount * 2.5);
            slaveSearchTextBox.Text = dSlaveSearchTimeout.ToString() + " s";
            usSlaveSearchCount = (ushort)((32768 / usSlaveChannelPeriod) * dSlaveSearchTimeout);
        }

        /// <summary>
        /// Connect to the Master USB device 
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void connectButton1_Click(object sender, EventArgs e)
        {
            //Check state of the button 
            if (connectButton1.Text.Equals("Connect Device"))
            {
                PrintMaster("Connecting Device..." + Environment.NewLine);
                try
                {
                    //Create a new instance of an ANT device if AP2 or AT3 detected
                    theMasterDevice = new ANT_Device(ucUSBPort1, usBaudRateAP2AP3);
                    EnableMasterUI(); //Display device capabilities
                }
                catch (ANT_Exception) 
                {
                    try
                    {
                        //Create a new instance of an ANT device if AP1 detected
                        theMasterDevice = new ANT_Device(ucUSBPort1, usBaudRateAP1);
                        EnableMasterUI(); //Display device capabilities 
                    }

                    catch (ANT_Exception)
                    {
                        PrintMaster("There is no ANT device connected to selected USB port or it is already in use by another USB device" + Environment.NewLine);
                    }
                }
            }
            else 
            {
                MasterStartState(); //Set application to start state
                PrintMaster("Disconnecting Device..." + Environment.NewLine);
                ANT_Device.shutdownDeviceInstance(ref theMasterDevice); //Destroy instance of ANT device
                PrintMaster("Successfully Disconnected Device." + Environment.NewLine + Environment.NewLine);
            }

        }

        /// <summary>
        /// Connect to the Slave USB device 
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void connectButton2_Click(object sender, EventArgs e)
        {
            //Check state of the button 
            if (connectButton2.Text.Equals("Connect Device"))
            {
                PrintSlave("Connecting Device..." + Environment.NewLine);
                try
                {
                    //Create a new istance of an ANT device if AP2 or AT3 detected
                    theSlaveDevice = new ANT_Device(ucUSBPort2, usBaudRateAP2AP3);
                    EnableSlaveUI();//Display device capabilities 
                    bAP1 = false; //The device is not an AP1 device
                    if (theSlaveDevice.EnableLED(true, 500)) //If the device has LED's, it is an AT3
                        bAT3 = true;
                }
                catch (ANT_Exception)
                {
                    try
                    {
                        //Create a new istance of an ANT device if AP1 detected
                        theSlaveDevice = new ANT_Device(ucUSBPort2, usBaudRateAP1);
                        EnableSlaveUI();//Display device capabilities 
                        bAP1 = true; //The device is an AP1 device
                    }
                    catch (ANT_Exception)
                    {
                        PrintSlave("There is no ANT device connected to selected USB port or it is already in use by another USB device" + Environment.NewLine);
                    }
                }
            }
            else
            {
                SlaveStartState(); //Set application to start state
                PrintSlave("Disconnecting Device..." + Environment.NewLine);
                ANT_Device.shutdownDeviceInstance(ref theSlaveDevice);//Destroy the instance of ANT device
                PrintSlave("Successfully Disconnected Device." + Environment.NewLine + Environment.NewLine);
            }

        }

        /// <summary>
        /// Configure the Master ANT device and open the Master channel 
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void openButton1_Click(object sender, EventArgs e)
        {
            //Check state of the button 
            if (openButton1.Text.Equals("Open Channel"))
            {
                try
                {
                    PrintMaster("Resetting Module..." + Environment.NewLine);
                    theMasterDevice.ResetSystem(); //Soft reset 
                    System.Threading.Thread.Sleep(500); //Give a 500ms delay after the reset

                    //Try setting the network key
                    if (!SetMasterNetworkKey())
                        throw new Exception("The network key is invalid"); //Throw exception if the Key is incomplete
                    if (theMasterDevice.setNetworkKey(ucNetworkNo, aucMasterNetworkKey, 500))
                        PrintMaster("Network Key Set : {" + String.Format("{0:X}, {1:X}, {2:X}, {3:X}, {4:X}, {5:X}, {6:X}, {7:X}", aucMasterNetworkKey[0], aucMasterNetworkKey[1], aucMasterNetworkKey[2], aucMasterNetworkKey[3], aucMasterNetworkKey[4], aucMasterNetworkKey[5],aucMasterNetworkKey[6],aucMasterNetworkKey[7]) + "}"+Environment.NewLine);
                    else
                        throw new Exception("Set Network Key operation failed"); //Throw excpetion if setting key failed

                    //Try assigning the channel type
                    if (_channel0.assignChannel(_masterChannelType, ucNetworkNo, 500))
                        PrintMaster("Channel Assigned" + Environment.NewLine);
                    else
                        throw new Exception("Channel Assignment operation failed"); //Throw exception of assigning channel failed

                    //Try setting the channel ID
                    if (_channel0.setChannelID(usMasterNo, false, ucMasterType, ucMasterTrans, 500))
                        PrintMaster("Channel ID Set: " + usMasterNo.ToString() + ", " + ucMasterType.ToString() + ", " + ucMasterTrans.ToString() + Environment.NewLine);
                    else
                        throw new Exception("Set Channel ID operation failed"); //Throw exception of setting ID failed

                    //Try setting the RF frequency
                    if (_channel0.setChannelFreq(ucMasterFreq1, 500))
                        PrintMaster("Radio Frequency Set: " + ucMasterFreq1.ToString() + Environment.NewLine);
                    else
                        throw new Exception("Set RF Frequency operation failed"); //Throw exception if setting RF Frequency failed

                    //Try setting the Channel Period
                    ushort usSeconds = ushort.Parse(masterSeconds.Text); //Temporary variable, holds value of Channel Period
                    if (_channel0.setChannelPeriod(usSeconds, 500))
                        PrintMaster("Channel Period Set: " + masterSeconds.Text + Environment.NewLine);
                    else
                        throw new Exception("Set Channel Period operation failed"); //Throw exception if set channel period failed

                    //Try opening the channel once everything has been set
                    PrintMaster("Opening Channel..." + Environment.NewLine);
                    if (_channel0.openChannel(500))
                    {
                        PrintMaster("Channel Opened Successfully" + Environment.NewLine + Environment.NewLine);
                        MasterConnectedChannelOpen(); //Set application to this state
                        _channel0.sendAcknowledgedData(aucTxBuffer); //Send the first acknowledged message 
                        currentRFTextBox1.Text = ucMasterFreq1.ToString(); //Display the current RF
                        //Set the allowable search count
                        usMasterSearchCount = (ushort)((double)(dMasterChannelFrequency) * (double)(dMasterSearchTimeout / 3));
                    }

                    else
                        throw new Exception("Open Channel operation failed"); //Throw exception if open channel failed
                }
                catch (Exception ex)
                {
                    //Display message box with the type of error
                    MessageBox.Show("Configuration Failed: " + ex.Message,"Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    //Return so the error may be fixed if possible 
                    return;
                }
            }

            else
            {
                _channel0.closeChannel(500); //Close the channel 
                MasterConnectedChannelClosed(); //Set application to this state 
            }
        }
        
        /// <summary>
        /// Configure the Slave ANT device and open the Slave channel 
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void openButton2_Click(object sender, EventArgs e)
        {
            //Check state of the button 
            if (openButton2.Text.Equals("Open Channel"))
            {
                try
                {
                    PrintSlave("Resetting Module..." + Environment.NewLine);
                    theSlaveDevice.ResetSystem(); //Soft reset 
                    System.Threading.Thread.Sleep(500); //Give a 500ms delay after the reset

                    //Try setting the network key
                    if (!SetSlaveNetworkKey())
                        throw new Exception("The network key is invalid"); //Throw exception if the key is incomplete
                    if (theSlaveDevice.setNetworkKey(ucNetworkNo, aucSlaveNetworkKey, 500))
                        PrintSlave("Network Key Set : {" + String.Format("{0:X}, {1:X}, {2:X}, {3:X}, {4:X}, {5:X}, {6:X}, {7:X}", aucSlaveNetworkKey[0], aucSlaveNetworkKey[1], aucSlaveNetworkKey[2], aucSlaveNetworkKey[3], aucSlaveNetworkKey[4], aucSlaveNetworkKey[5], aucSlaveNetworkKey[6], aucSlaveNetworkKey[7]) + "}" + Environment.NewLine);
                    else
                        throw new Exception("Set Network Key operation failed"); //Throw exception if set network key failed

                    //Try setting the channel type 
                    if (_channel1.assignChannel(_slaveChannelType, ucNetworkNo, 500))
                        PrintSlave("Channel Assigned" + Environment.NewLine);
                    else
                        throw new Exception("Channel Assignment operation failed"); //Throw exception if assign channel failed

                    //Try setting the channel ID
                    if (_channel1.setChannelID(usSlaveNo, false, ucSlaveType, ucSlaveTrans, 500))
                        PrintSlave("Channel ID Set: " + usSlaveNo.ToString() + ", " + ucSlaveType.ToString() + ", " + ucSlaveTrans.ToString() + Environment.NewLine);
                    else
                        throw new Exception("Set Channel ID operation failed"); //Throw exception if set channel ID failed

                    //Try setting the RF frequency
                    if (_channel1.setChannelFreq(ucSlaveRF, 500))
                        PrintSlave("Radio Frequency Set: " + ucSlaveRF.ToString() + Environment.NewLine);
                    else
                        throw new Exception("Set RF Frequency operation failed"); //Throw exception if set RF Frequency failed

                    //Try setting the channel period
                    ushort usSeconds = ushort.Parse(slaveSeconds.Text); //Temporary variable holds value of channel period
                    if (_channel1.setChannelPeriod(usSeconds, 500))
                        PrintSlave("Channel Period Set: " + slaveSeconds.Text + Environment.NewLine);
                    else
                        throw new Exception("Set Channel Period operation failed"); //Throw exception if set channel period failed

                    //Try setting the slave search timeout 
                    if (_channel1.setChannelSearchTimeout((byte)slaveSearchUpDown.Value, 500))
                        PrintSlave("Search Timeout Set" + Environment.NewLine);
                    else
                        throw new Exception("Search Timeout operation failed"); //Throw exception if set search timeout failed

                    //Try setting Low Priority search time to 0 (only applies to AP2/AT3)
                    if (!bAP1)
                    {
                        if (!_channel1.setLowPrioritySearchTimeout(0, 500))
                            throw new Exception("Low Priority Search Timeout operation failed"); //Throw exception if set low priority search failed
                    }

                    //Try opening the channel
                    PrintSlave("Opening Channel..." + Environment.NewLine);
                    if (_channel1.openChannel(500))
                    {
                        PrintSlave("Channel Opened Successfully" + Environment.NewLine + Environment.NewLine + "Searching on RF: " + ucSlaveRF.ToString()+ "..."+Environment.NewLine);
                        SlaveConnectedChannelOpen(); //Set the application to this state
                        currentRFTextBox2.Text = ucSlaveFreq1.ToString(); //Display the current RF 
                    }
                    else
                        throw new Exception("Open Channel operation failed"); //Throw exception if open channel failed

                }
                catch (Exception ex)
                {
                    //Display message box with the type of error
                    MessageBox.Show("Configuration Failed: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    //Return so the error may be fixed if possible 
                    return;
                }
            }
            else
            {
                bSlaveClosed = true; //Indicate that the close button was clicked
                _channel1.closeChannel(500); //Close the channel
                SlaveConnectedChannelClosed();//Set application to this state 
            }
        }

        /// <summary>
        /// Clear the application display
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void clearButton1_Click(object sender, EventArgs e)
        {
            masterTextBox.Clear();
        }
        private void clearButton2_Click(object sender, EventArgs e)
        {
            slaveTextBox.Clear();
        }

        /// <summary>
        /// Handle Master ANT Channel Response Events
        /// </summary>
        /// <param name="response">Master ANT Response</param>
        private void MasterChannelResponse(ANT_Response response)
        {
            //Checks the message ID received 
            switch ((ANT_ReferenceLibrary.ANTMessageID)response.responseID)
            {
                case ANT_ReferenceLibrary.ANTMessageID.RESPONSE_EVENT_0x40:
                    {
                        //Checks the event ID received
                        switch (response.getChannelEventCode())
                        {
                            case ANT_ReferenceLibrary.ANTEventID.EVENT_TX_0x03:
                                {
                                    break;
                                }
                            //Occurs when an acknowledge message is successfully sent
                            case ANT_ReferenceLibrary.ANTEventID.EVENT_TRANSFER_TX_COMPLETED_0x05:
                                {
                                    //Increment the last byte of the message
                                    aucTxBuffer[7]++;
                                    //Transmit the acknowledge data in the buffer. Must be acknowledge data for frequency agility to work 
                                    _channel0.sendAcknowledgedData(aucTxBuffer);
                                    PrintMaster("Tx: (" + response.antChannel.ToString() + ")" + BitConverter.ToString(aucTxBuffer) + Environment.NewLine);
                                    //Set mastercounter to 1, no need to count if no message has been missed
                                    usMasterCounter = 1;
                                    break;
                                }
                            //Occurs when no acknowledge message is received from slave
                            case ANT_ReferenceLibrary.ANTEventID.EVENT_TRANSFER_TX_FAILED_0x06:
                                {
                                    PrintMaster("Ack failed on RF: " + ucMasterRF.ToString() + " MasterCounter: " + usMasterCounter + Environment.NewLine);
                                    //Check if the number of miss messages is less than 
                                    //the total number of allowable missed messages
                                    if (usMasterCounter < usMasterSearchCount)
                                        usMasterCounter++; //Increment the number of messages missed
                                    else
                                    {
                                        //Jump to a new frequency
                                        HandleMasterInterference();
                                        //Set the counter to 1. Master starts counting missed messages
                                        //on new frequency
                                        usMasterCounter = 1;
                                    }
                                    //Send acknowledge message 
                                    _channel0.sendAcknowledgedData(aucTxBuffer);
                                    break;
                                }
                            //Occurs when the channel is closed 
                            case ANT_ReferenceLibrary.ANTEventID.EVENT_CHANNEL_CLOSED_0x07:
                                {
                                    PrintMaster("Channel Closed" + Environment.NewLine + Environment.NewLine);
                                    break;
                                }
                            default:
                                {
                                    //Handle any events that weren't acknowledge above
                                    PrintMaster("Unknown Channel Event " + ((byte)response.getChannelEventCode()).ToString() + Environment.NewLine);
                                    break;
                                }
                        }
                        break;
                    }
                default:
                    {
                        //Handle any messages that weren't acknowledged above
                        PrintMaster("Unknown Channel (" + response.antChannel.ToString() + ") Message " + response.responseID + Environment.NewLine);
                        break;
                    }

            }
        }

        /// <summary>
        /// Handle Slave ANT Channel Response Events
        /// </summary>
        /// <param name="response">Slave ANT Response</param>
        private void SlaveChannelResponse(ANT_Response response)
        {
            //Checks message ID received
            switch ((ANT_ReferenceLibrary.ANTMessageID)response.responseID)
            {
                case ANT_ReferenceLibrary.ANTMessageID.RESPONSE_EVENT_0x40:
                    {
                        //Checks event ID received
                        switch (response.getChannelEventCode())
                        {
                            //Occurs in the case of a search timeout
                            case ANT_ReferenceLibrary.ANTEventID.EVENT_RX_SEARCH_TIMEOUT_0x01:
                                {
                                    ucSlaveSearchTimeoutCount++; //Increment the search timeout counter
                                    //Check if the total number of consecutive search timeouts
                                    //is less than the total number of allowable search timeouts (6)
                                    if (ucSlaveSearchTimeoutCount >= 6)
                                    {
                                        //If the slave has searched 6 times
                                        //Close the channel
                                        bSlaveClosed = true;
                                        ucSlaveRF = ucSlaveFreq1;
                                        //The AP1 and AT3 devices need to be told explicitly
                                        //to close the channel because frequency agility
                                        //is not built into these devices
                                        if (bAP1 || bAT3)
                                            goto case ANT_ReferenceLibrary.ANTEventID.EVENT_CHANNEL_CLOSED_0x07;
                                        break;
                                    }
                                    //Set ucSlaveReceived to 0 to detected the next acknowledge message received
                                    ucSlaveReceived = 0;
                                    PrintSlave("Search Timeout on channel" + Environment.NewLine);
                                    //Jump to new frequency
                                    HandleSlaveInterference();
                                    //The AP1 and AT3 devices need to be told explicitly to
                                    //close the channel because frequency agility
                                    //is not built into these devices
                                    if (bAP1 || bAT3)
                                        goto case ANT_ReferenceLibrary.ANTEventID.EVENT_CHANNEL_CLOSED_0x07;
                                    break;
                                }
                            //Occurs if the device fails to receive an 
                            //expeceted message at the channel frequency 
                            case ANT_ReferenceLibrary.ANTEventID.EVENT_RX_FAIL_0x02:
                                {
                                    PrintSlave("Rx Fail " + Environment.NewLine);
                                    usNoConnection++; //Increment the RX_FAIL counter if an RX_FAIL has occurred
                                    //AP1 does not support EVENT_RX_FAIL_GO_TO_SEARCH
                                    // so it is necessary to count Rx fails 
                                    if (usNoConnection < usSlaveMissedRx)
                                    {
                                        ucSlaveSearchTimeoutCount = 0;
                                        break;
                                    }
                                    else
                                    {
                                        //If maximum number of RX_FAIL events have occurred
                                        //Explicitly tell the AP1 module to go to search 
                                        //Set the search timeout count to 0, because RX_FAIL
                                        //events occur only after a device has been receiving, 
                                        //then stops receiving
                                        ucSlaveSearchTimeoutCount = 0;
                                        goto case ANT_ReferenceLibrary.ANTEventID.EVENT_RX_FAIL_GO_TO_SEARCH_0x08;
                                    }
                                }
                            //Occurs if the maximum amount of RX_Fail events has occurred
                            case ANT_ReferenceLibrary.ANTEventID.EVENT_RX_FAIL_GO_TO_SEARCH_0x08:
                                {
                                    usNoConnection = 0; //Set RX_FAIL count to 0
                                    //Check if the slave has received any data from a master prior to dropping to search 
                                    if (ucSlaveReceived != 0)
                                        //Jump frequencies
                                        HandleSlaveInterference();
                                    PrintSlave("Searching on RF " + ucSlaveRF.ToString() + "..." + Environment.NewLine);
                                    break;
                                }
                            //Occurs when the channel is closed
                            case ANT_ReferenceLibrary.ANTEventID.EVENT_CHANNEL_CLOSED_0x07:
                                {
                                    //Check if the 'Close Channel' button was clicked
                                    //or if 6 consecutive Search timeout events have occurred
                                    if (!bSlaveClosed)
                                    {
                                        _channel1.openChannel(); //Reopen the channel and drop to search
                                        goto case ANT_ReferenceLibrary.ANTEventID.EVENT_RX_FAIL_GO_TO_SEARCH_0x08;
                                    }
                                    PrintSlave("Channel Closed" + Environment.NewLine + Environment.NewLine);
                                    SlaveConnectedChannelClosed(); //Set the application to this state
                                    break;
                                }
                                //Handles any events that weren't acknowledged above
                            default:
                                {
                                    PrintSlave("Unknown Channel (" + response.antChannel.ToString() + ") Event " + ((byte)response.getChannelEventCode()).ToString());
                                    break;
                                }
                        }
                        break;
                    }
                //Occurs when acknowledged is received from a master 
                case ANT_ReferenceLibrary.ANTMessageID.ACKNOWLEDGED_DATA_0x4F:
                    {
                        //Check if the slave has received data previously on this frequency
                        if (ucSlaveReceived == 0)
                            //If no data was received before, indicate that there is now
                            ucSlaveReceived = 1;
                        usNoConnection = 0;//set RX_FAIL count to 0
                        PrintSlave("Rx: (" + response.antChannel.ToString() + ")" + BitConverter.ToString(aucTxBuffer) + Environment.NewLine);
                        break;
                    }
                //Handles any messages that weren't acknowledged above
                default:
                    {
                        PrintSlave("Unknown Channel (" + response.antChannel.ToString() + ") Message" + response.responseID + Environment.NewLine);
                        break;
                    }
            }
        }

        /// <summary>
        /// Handle Master frequency jump 
        /// </summary>
        private void HandleMasterInterference()
        {
            //Set case for switch statement by checking current RF 
            byte yFrequency = 0;
            if (ucMasterRF == ucMasterFreq1)
                yFrequency = 1;
            else if (ucMasterRF == ucMasterFreq2)
                yFrequency = 2;
            else if (ucMasterRF == ucMasterFreq3)
                yFrequency = 3;

            switch (yFrequency)
            {
                case 1:
                    {
                        //Change RF to second in list
                        ucMasterRF = ucMasterFreq2;
                        //Change the channel frequency to updated RF
                        _channel0.setChannelFreq(ucMasterRF);
                        //Change the value of current RF displayed on application
                        this.SetText(ucMasterRF.ToString());
                        break;
                    }
                case 2:
                    {
                        //Change RF to last in list
                        ucMasterRF = ucMasterFreq3;
                        //Change the channel frequency to updated RF
                        _channel0.setChannelFreq(ucMasterRF);
                        //Change the value of current RF displayed on application 
                        this.SetText(ucMasterRF.ToString());
                        break;
                    }
                case 3:
                    {
                        //Change RF to first in list
                        ucMasterRF = ucMasterFreq1;
                        //Change the channel frequency to update RF
                        _channel0.setChannelFreq(ucMasterRF);
                        //Change the value of current RF displayed in application
                        this.SetText(ucMasterRF.ToString());
                        break;
                    }
                default:
                    {
                        break;
                    }
            }
            return;
        }

        /// <summary>
        /// Handle Slave frequency jump
        /// </summary>
        private void HandleSlaveInterference()
        {
            //Set case for switch statement by checking current RF
            byte yFrequency = 0;
            if (ucSlaveRF == ucSlaveFreq1)
                yFrequency = 1;
            else if (ucSlaveRF == ucSlaveFreq2)
                yFrequency = 2;
            else if (ucSlaveRF == ucSlaveFreq3)
                yFrequency = 3;

            switch (yFrequency)
            {
                case 1:
                    {
                        //Change RF to second in list
                        ucSlaveRF = ucSlaveFreq2;
                        //Change channel frequency to updated RF
                        _channel1.setChannelFreq(ucSlaveRF);
                        //Change the current RF displayed on applicaiton
                        this.SetTextS(ucSlaveRF.ToString());
                        break;
                    }
                case 2:
                    {
                        //Change RF to third in list
                        ucSlaveRF = ucSlaveFreq3;
                        //Change channel frequency to updated RF
                        _channel1.setChannelFreq(ucSlaveRF);
                        //Change the current RF displayed on the application
                        this.SetTextS(ucSlaveRF.ToString());
                        break;
                    }
                case 3:
                    {
                        //Change RF to first in list
                        ucSlaveRF = ucSlaveFreq1;
                        //Change channel frequency to updated RF
                        _channel1.setChannelFreq(ucSlaveRF);
                        //Change the current RF displayed on the application
                        this.SetTextS(ucSlaveRF.ToString());
                        break;
                    }
                default:
                    {
                        break;
                    }
            }
            return;
        }

        /// <summary>
        /// Display a string in the application textbox
        /// </summary>
        /// <param name="msg">String to Display</param>
        private void PrintMaster(String msg)
        {
            //Displays in Master textbox only
            masterTextBox.BeginInvoke(new dPrint(masterTextBox.AppendText), msg);

        }
        private void PrintSlave(String msg)
        {
            //Displays in slave textbox only
            slaveTextBox.BeginInvoke(new dPrint(slaveTextBox.AppendText), msg);
        }

        /// <summary>
        /// Display ANT device capabilities
        /// </summary>
        private void EnableMasterUI()
        {

            PrintMaster("Device Connected Successfully!" + Environment.NewLine + Environment.NewLine);
            PrintMaster("Master Device Details:" + Environment.NewLine);
            //Create a string to hold the device capabilities 
            String sCapability = theMasterDevice.getDeviceCapabilities().printCapabilities(); 
            String subString = ""; //Create an empty string
            for (int i = 0; i < sCapability.Length; i++) //For loop to display each capability on new line
            {
                if (sCapability[i] != '\n')
                    subString += sCapability[i];
                else
                {
                    PrintMaster(subString + Environment.NewLine);
                    subString = "";
                }
            }
            MasterConnectedChannelClosed(); //Set applicaiton to this state
            PrintMaster(Environment.NewLine);

            _channel0 = theMasterDevice.getChannel(0);//Get ANT channel
            //Assign channel response to the ANT channel
            _channel0.channelResponse += new ANT_Channel.ChannelResponseHandler(MasterChannelResponse);
        }
        private void EnableSlaveUI()
        {
            connectButton2.Text = "Disconnect Device";
            PrintSlave("Device Connected Successfully!" + Environment.NewLine + Environment.NewLine);
            PrintSlave("Slave Device Details:" + Environment.NewLine);
            //Create a string to hold the device capabilities
            String sCapability = theSlaveDevice.getDeviceCapabilities().printCapabilities();
            String subString = ""; //Create an empty string 
            for (int i = 0; i < sCapability.Length; i++) //For loop to display each capability on new line
            {
                if (sCapability[i] != '\n')
                    subString += sCapability[i];
                else
                {
                    PrintSlave(subString + Environment.NewLine);
                    subString = "";
                }
            }
            SlaveConnectedChannelClosed();//Set application to this state
            PrintSlave(Environment.NewLine);

            _channel1 = theSlaveDevice.getChannel(1);//Get ANT Channel
            //Assign channel response to the ant channel
            _channel1.channelResponse += new ANT_Channel.ChannelResponseHandler(SlaveChannelResponse);
        }

        /// <summary>
        /// States for the application
        /// </summary>
        private void MasterStartState()
        {
            //Label buttons
            connectButton1.Text = "Connect Device";
            openButton1.Text = "Open Channel";


            //Disable buttons that do not apply
            clearButton1.Enabled = false;
            openButton1.Enabled = false;

            //Set default USB port
            ucUSBPort1 = 0;
            usbComboBox1.Text = "0";
            usbComboBox1.Enabled = true;

            //Allow period to be changed
            masterSeconds.Enabled = true;
            masterHz.Enabled = true;

            //Allow ID to be changed 
            deviceNo1.Enabled = true;
            deviceType1.Enabled = true;
            transType1.Enabled = true;
            networkKeyTextBox1.Enabled = true;

            //Allow frequency agility configuration to be changed
            freq11.Enabled = true;
            freq21.Enabled = true;
            freq31.Enabled = true;

            //Allow search timeout to be changed
            masterSearchUpDown.Enabled = true;

            //Set Number of missed messages to 1
            usMasterCounter = 1;

            //Clear the master textbox
            masterTextBox.Clear();
        }
        private void SlaveStartState()
        {
            //Label buttons
            connectButton2.Text = "Connect Device";
            openButton2.Text = "Open Channel";

            //Disable buttons that do not apply
            clearButton2.Enabled = false;
            openButton2.Enabled = false;

            //Set USB port 
            ucUSBPort2 = 1;
            usbComboBox2.Text = "1";
            usbComboBox2.Enabled = true;

            //Allow channel period to be changed
            slaveSeconds.Enabled = true;
            slaveHz.Enabled = true;

            //Allow the channel ID to be changed
            deviceNo2.Enabled = true;
            deviceType2.Enabled = true;
            transType2.Enabled = true;
            networkKeyTextBox2.Enabled = true;

            //Allow the frequency agility configuration to be changed
            freq12.Enabled = true;
            freq22.Enabled = true;
            freq32.Enabled = true;


            //Allow the search timeout to be changed
            slaveSearchUpDown.Enabled = true;

            //Clear the slave textbox
            slaveTextBox.Clear();

            //The channel has not been closed
            bSlaveClosed = false;
        }
        private void MasterConnectedChannelClosed()
        {
            //Label buttons
            connectButton1.Text = "Disconnect Device";
            openButton1.Text = "Open Channel";

            //Enabled buttons that apply
            clearButton1.Enabled = true;
            openButton1.Enabled = true;

            //Disable changing USB port number
            usbComboBox1.Enabled = false;

            //Allow channel configuration data to be changed
            masterSeconds.Enabled = true;
            masterHz.Enabled = true;
            deviceNo1.Enabled = true;
            deviceType1.Enabled = true;
            transType1.Enabled = true;
            networkKeyTextBox1.Enabled = true;
            freq11.Enabled = true;
            freq21.Enabled = true;
            freq31.Enabled = true;
            masterSearchUpDown.Enabled = true;

            //Set the mastercounter to 1
            usMasterCounter = 1;
            ucMasterRF = ucMasterFreq1;

        }
        private void SlaveConnectedChannelClosed()
        {
            //Label Buttons 
            if (this.connectButton2.InvokeRequired)
            {
                dSetButtonText d = new dSetButtonText(SlaveConnectedChannelClosed);
                this.Invoke(d);
            }
            else
                openButton2.Text = "Open Channel";
                connectButton2.Text = "Disconnect Device";

            //Enable buttons that apply
            clearButton2.Enabled = true;
            openButton2.Enabled = true;

            //Disable USB port number from being changed
            usbComboBox2.Enabled = false;

            //Allow channel configuration data to be changed
            slaveSeconds.Enabled = true;
            slaveHz.Enabled = true;
            deviceNo2.Enabled = true;
            deviceType2.Enabled = true;
            transType2.Enabled = true;
            networkKeyTextBox2.Enabled = true;
            freq12.Enabled = true;
            freq22.Enabled = true;
            freq32.Enabled = true;
            slaveSearchUpDown.Enabled = true;

            //Search search timeout counter to 0
            ucSlaveSearchTimeoutCount = 0;
            bSlaveClosed = false;
            ucSlaveRF = ucSlaveFreq1;
        }
        private void MasterConnectedChannelOpen()
        {
            //Label buttons
            connectButton1.Text = "Disconnect Device";
            openButton1.Text = "Close Channel";

            //Enable buttons that apply
            clearButton1.Enabled = true;
            openButton1.Enabled = true;

            //Disable changing of channel parameters
            usbComboBox1.Enabled = false;
            masterSeconds.Enabled = false;
            masterHz.Enabled = false;
            deviceNo1.Enabled = false;
            deviceType1.Enabled = false;
            transType1.Enabled = false;
            networkKeyTextBox1.Enabled = false;
            freq11.Enabled = false;
            freq21.Enabled = false;
            freq31.Enabled = false;
            masterSearchUpDown.Enabled = false;
        }
        private void SlaveConnectedChannelOpen()
        {
            //Label buttons
            connectButton2.Text = "Disconnect Device";
            openButton2.Text = "Close Channel";

            //Enable buttons that apply
            clearButton2.Enabled = true;
            openButton2.Enabled = true;

            //Disable changing channel parameters
            usbComboBox2.Enabled = false;
            slaveSeconds.Enabled = false;
            slaveHz.Enabled = false;
            deviceNo2.Enabled = false;
            deviceType2.Enabled = false;
            transType2.Enabled = false;
            networkKeyTextBox2.Enabled = false;
            freq12.Enabled = false;
            freq22.Enabled = false;
            freq32.Enabled = false;
            slaveSearchUpDown.Enabled = false;
        }

        /// <summary>
        /// Allow log files in the application
        /// </summary>
        private void EnableLogging()
        {
            //Enable log and make sure it was successful
            if (!ANT_Common.enableDebugLogs())
            {
                MessageBox.Show("Enabling logs failed. Communication will not be recorded", "Log File Failure");
                return;
            }
            //Catch any file errors that might exist
            try
            {
                //Create the log directory if it doesn't already exist
                if (!System.IO.Directory.Exists(logDirectory))
                    System.IO.Directory.CreateDirectory(logDirectory);
                //Set the library to save logs in the log directory 
                if (!ANT_Common.setDebugLogDirectory(logDirectory + "\\"))
                    throw new Exception("Could not set log file directory");
            }
            catch (Exception)
            {
                MessageBox.Show("Enabling log files failed, could not set log directory, ensure application is running with appropriate permission to creat files. Communication will not be recorded.", "Log File Failure");
                return;
            }
        }

        /// <summary>
        /// Allow for cross thread calls 
        /// </summary>
        /// <param name="s">String to display</param>
        private void SetText(string s)
        {
            //Check if caller is on a different thread than what control was created on
            if (this.currentRFTextBox1.InvokeRequired)
            {
                dSetTextCallback d = new dSetTextCallback(SetText);
                this.Invoke(d, new object[] { s });
            }
            else
            {
                this.currentRFTextBox1.Text = s;
            }
        }
        private void SetTextS(string s)
        {
            //Check if caller is on a different thread than what control was created on
            if (this.currentRFTextBox2.InvokeRequired)
            {
                dSetTextCallback d = new dSetTextCallback(SetTextS);
                this.Invoke(d, new object[] { s });
            }
            else
            {
                this.currentRFTextBox2.Text = s;
            }
        }

        /// <summary>
        /// Close USB connections when application is exited
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Event arguments</param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            //This only needs to be done if the device has not already been disconnected 
            //when the application is closed
            if (openButton1.Text == "Disconnect Device")
            {
                theMasterDevice.ResetSystem();
                theSlaveDevice.ResetSystem();
                System.Threading.Thread.Sleep(500);
                //Destroy instances of ANT devices
                ANT_Device.shutdownDeviceInstance(ref theMasterDevice);
                ANT_Device.shutdownDeviceInstance(ref theSlaveDevice);
            }
        }

        /// <summary>
        /// Allow only hexadecimal values for the network key
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Key Event arguments</param>
        private void networkKeyTextBox1_KeyDown(object sender, KeyEventArgs e)
        {
            //Check what key was pressed, supress any that do not correspond to a Hex number
            if (e.KeyCode > Keys.F && e.KeyCode <= Keys.Z && !(e.Control) && !(e.Alt))
                e.SuppressKeyPress = true;
        }
        private void networkKeyTextBox2_KeyDown(object sender, KeyEventArgs e)
        {
            //Check what key was pressed, supress any that do not correspond to a hex number
            if (e.KeyCode > Keys.F && e.KeyCode <= Keys.Z && !(e.Control) && !(e.Alt))
                e.SuppressKeyPress = true;
        }

        /// <summary>
        /// Set the Network key from user input
        /// </summary>
        /// <returns>true if network key enter is valid, false otherwise</returns>
        private bool SetMasterNetworkKey()
        {
            if (networkKeyTextBox1.MaskFull) //Check if the masked textbox is full 
            {
                string s = networkKeyTextBox1.Text; //Holds what was typed in the textbox
                string t = ""; //String to step through network key
                int newbyte = 0;//Set to 1 when part of one byte of the network key has been found
                int temp2 = 0;//Integer for adding the two parts of the byte together
                byte value = 0;//Used to set decimal value for hex numbers 0xA, 0xB, 0xC, 0xD, 0xE, 0xF

                for(int i = 22, k = 7; i > 0;i--, k--) //Start the index from the last character of the string, and last byte of the network key
                {
                    byte temp = 0; //Store the value of the byte in decimal
                    t = s.Substring(i, 1); //Get the character of the string at the index i
                        while(!t.Contains("-")) //Decrement the index and try again
                        {
                            if(t.Equals("A")) //Set value to 10 if string at index is A
                                value = 10;
                            if (t.Equals("B")) //Set value to 11 if string at index is B
                                value = 11;
                            if (t.Equals("C")) //Set value to 12 if string at index is C
                                value = 12;
                            if (t.Equals("D")) //Set value to 13 if string at index is D
                                value = 13;
                            if (t.Equals("E")) //Set value to 14 if string at index is E
                                value = 14;
                            if (t.Equals("F")) //Set value to 15 if string at index is F
                                value = 15;

                            if(newbyte == 0)
                            {
                                if(value !=0) //If a hex number between 0xA and 0xF was found, set temp to its decimal value
                                {
                                    temp = value;
                                    value = 0;
                                }
                                else
                                    temp = byte.Parse(t);//Set temp to the decimal value of the number found in the string at index i
                                newbyte++;
                            }
                            else
                            {
                                if(value != 0)//If a hex number between 0xA and 0xF was found, set temp2 to (decimal value * 2^4)  
                                {
                                    temp2 = (int)value << 4;
                                    value = 0;
                                }
                                else
                                    temp2 = int.Parse(t) << 4; //Set temp2 to (decimal value * 2^4)
                                temp += (byte)temp2; //Get the decimal value of the hexidecimal byte by adding temp and temp2
                                aucMasterNetworkKey[k] = temp; //Set the network key at index k to the decimal value of the byte in temp
                                newbyte = 0; //Indicate that this byte of the network key has been set, a new byte can be started
                                }
                            if (i == 0) //Ensure index does not go below 0
                                break;
                            i--; //Decrement the counter to move through the string
                            t = s.Substring(i, 1); //Set the temporary string to the string at the new index so you can check for '-'
                        }
                }
                return true; //After setting the network key on a full mask, return true
            }
            else
                return false;//If the mask wasn't full, return false
        }
        private bool SetSlaveNetworkKey()
        {
            if (networkKeyTextBox2.MaskFull) //Check if the masked textbox is full 
            {
                string s = networkKeyTextBox2.Text; //Holds what was typed in the textbox
                string t = ""; //String to step through network key
                int newbyte = 0; //Set to 1 when part of one byte of the network key has been found
                int temp2 = 0; //Integer for adding the two parts of the hex number together
                byte value = 0; //Used to set decimal value for hex numbers 0xA, 0xB, 0xC, 0xD, 0xE, 0xF

                for (int i = 22, k = 7; i > 0; i--, k--) //Start the index from the last character of the string, and last byte of the network key
                {
                    byte temp = 0; //Temporary variable to store the value of the byte in decimal
                    t = s.Substring(i, 1); //Get the character of the string at the index i
                    while (!t.Contains("-")) //Decrement the index and try again
                    {
                        if (t.Equals("A")) //Set value to 10 if string at index is A
                            value = 10;
                        if (t.Equals("B")) //Set value to 11 if string at index is B
                            value = 11;
                        if (t.Equals("C")) //Set value to 12 if string at index is C
                            value = 12;
                        if (t.Equals("D")) //Set value to 13 if string at index is D
                            value = 13;
                        if (t.Equals("E")) //Set value to 14 if string at index is E
                            value = 14;
                        if (t.Equals("F")) //Set value to 15 if string at index is F
                            value = 15;

                        if (newbyte == 0)
                        {
                            if (value != 0) //If a hex number between 0xA and 0xF was found, set temp to its decimal value
                            {
                                temp = value;
                                value = 0;
                            }
                            else
                                temp = byte.Parse(t); //Set temp to the decimal value of the number found in the string at index i
                            newbyte++;
                        }
                        else
                        {
                            if (value != 0) //If a hex number between 0xA and 0xF was found, set temp2 to (decimal value * 2^4)  
                            {
                                temp2 = (int)value << 4;
                                value = 0;
                            }
                            else
                                temp2 = int.Parse(t) << 4; //set temp2 to the (decimal value * 2^4)
                            temp += (byte)temp2; //Get the decimal value of the hexidecimal byte by adding temp and temp2
                            aucSlaveNetworkKey[k] = temp; //Set the network key at index k to the decimal value of the byte in temp
                            newbyte = 0; //Indicate that this byte of the network key has been set, a new byte can be started
                        }
                        if (i == 0) //Ensure index does not go below 0
                            break;
                        i--; //Decrement the counter to move through the string
                        t = s.Substring(i, 1); //Set the temporary string to the string at the new index so you can check for '-'
                    }
                }
                return true; //After setting the network key on a full mask, return true
            }
            else
                return false; //If the mask wasn't full, return false
        }        
            
    }
    
 
}

    

